"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const node_1 = require("vscode-languageserver/node");
const ember_meta_explorer_1 = require("ember-meta-explorer");
const estree_utils_1 = require("../estree-utils");
const logger_1 = require("../utils/logger");
const traverse_1 = require("@babel/traverse");
class JSDocumentSymbolProvider {
    constructor() {
        this.extensions = ['.js'];
    }
    process(content) {
        const symbols = [];
        try {
            const ast = (0, ember_meta_explorer_1.parseScriptFile)(content);
            (0, traverse_1.default)(ast, {
                Property(path) {
                    const node = path.node;
                    const symbol = node_1.SymbolInformation.create(node.key.name, node_1.SymbolKind.Property, (0, estree_utils_1.toLSRange)(node.key.loc));
                    symbols.push(symbol);
                },
            });
        }
        catch (e) {
            (0, logger_1.logDebugInfo)('symbolprovider:script:error', e, e.toString(), e.stack);
        }
        return symbols;
    }
}
exports.default = JSDocumentSymbolProvider;
//# sourceMappingURL=js-document-symbol-provider.js.map